import { InputError } from '@roots/bud-support/errors';
/**
 * Returns {@link Bud} instance from {@link Bud.children}
 *
 * @remarks
 * An optional {@link tap} function can be provided to configure the {@link Bud} instance.
 *
 * @example
 * ```js
 * bud.get(label)
 * ```
 */
export const get = function (label, tap) {
    const { isRoot, root, warn } = this;
    !isRoot &&
        warn(`not root instance. returning from the context of ${root.label}`);
    if (!root.children?.[label]) {
        throw new InputError(`bud.get: bud.children['${label}'] is undefined`, {
            details: `There are no instances with the label "${label}" associated with this instance`,
            thrownBy: `bud.get`,
        });
    }
    root.success(`returning child instance:`, label);
    return root.children[label];
};
